/*
 * IDLflex -- A flexible, generic code generator for CORBA IDL
 *
 * Copyright (C) 2000  Hans Reiser, AspectIX Research Team
 *                     University of Erlangen-Nuernberg, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 * The full text of the license can be found in the LICENSE.txt file.
 *
 * $Id: IBMXMLparser.java,v 1.2 2000/12/15 15:32:25 fzhauck Exp $
 */

package org.aspectix.IDLflex;

import org.xml.sax.InputSource;
import org.w3c.dom.Element;
import org.aspectix.IDLflex.XMLparser;
import java.io.*;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/** XML parser implemention which uses the IBM XML4J (Xerces) XML parser.
 */
public class IBMXMLparser implements XMLparser
{
    String configPath;

    /** As Xerces by default just searches for Mapping.dtd in the
     *  current directory, this EntityResolver when lookup up the
     *  DTD file tries a) to find it in the same directory as the
     *  XML docuemnt b) to find it in $(IDLflexHome)/mapping/.
     */
     class dtdEntityResolver implements EntityResolver {
	public InputSource resolveEntity(String pubID, String sysID) {
	    if(sysID.endsWith("Mapping.dtd")) {
		try {
		    return new InputSource(new FileReader(
                          new File(configPath, "Mapping.dtd")));
		}
		catch(Exception e) {};
		try {
		    String IDLflexHome = java.lang.System.getProperty("IDLflexHome",".");
		    return new InputSource(new FileReader(
			  new File(IDLflexHome+"/mapping", "Mapping.dtd")));
		}
		catch(Exception e) {};
	    }
	    return null;
	}
    }

    /** As the default behavior is to ignor errors and warnings, this
     *  ErrorHandler is used to create an Exception for all errors
     *  and a message on stderr for warnings. */
    class myErrorHandler implements ErrorHandler {
	public void error(SAXParseException exception) 
	    throws SAXParseException
	{
	    throw exception;
	}
	public void fatalError(SAXParseException exception) 
	    throws SAXParseException
	{
	    throw exception;
	}
	public void warning(SAXParseException exception) {
	    System.err.println(exception);
	}
    }

    /** parser initialization and invocation */
    public Element parse(String configFile) throws Exception
    {
	org.xml.sax.InputSource input;

	File cfgFile = new File(configFile);
	configPath = cfgFile.getParent();

	FileReader freader = new FileReader(cfgFile);
	input = new org.xml.sax.InputSource( freader );
	org.apache.xerces.parsers.DOMParser prs =
	    new org.apache.xerces.parsers.DOMParser();

	prs.setEntityResolver(new dtdEntityResolver());
	prs.setErrorHandler(new myErrorHandler());
	prs.setFeature("http://xml.org/sax/features/validation",true);

	prs.parse(input);

	return prs.getDocument().getDocumentElement();
    }
}
