/*
 * IDLflex -- A flexible, generic code generator for CORBA IDL
 *
 * Copyright (C) 2000  Hans Reiser, AspectIX Research Team
 *                     University of Erlangen-Nuernberg, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 * The full text of the license can be found in the LICENSE.txt file.
 *
 * $Id: StructObj.java,v 1.4 2001/09/07 09:20:45 reiser Exp $
 */

package org.aspectix.IDLflex.IRObj;

import org.omg.CORBA.*;
import org.aspectix.IDLflex.IDLflexException;
import java.util.Vector;

/* implementation of an IDL structure
 *<ul>
 * <li><em>getContentList("ELEMENTS")</em>
 *     returns the list of structure member elements
 *</ul>
 */
public class StructObj extends ContainerObj {
    Vector elements;
    public StructObj(IRObject iro) {
	super(iro);  // set object reference and name
    }
    public void init_members() throws IDLflexException {
	super.init_members();
	if(referenceObject!=null) {
	    StructDef sd = StructDefHelper.narrow(referenceObject);
	    StructMember[] smlist = sd.members();
	    elements = new Vector();
	    for(int i=0; i<smlist.length; i++) {
		/*
		System.err.println("StructObj: "+i+"\n"+smlist[i]+"\n"+
				   smlist[i].type_def);
		*/
		IDLObject idlo = new StructMemberObj( smlist[i] );
		idlo.init_members();
		idlo.defined_in=this.defined_in;
		elements.addElement(idlo);
	    }
	}
    }
    public IDLObject[] getContentList(String spec) throws IDLflexException {
	if(spec.equalsIgnoreCase("elements")) 
	    return VectorToIDLObjects(elements);
	else return super.getContentList(spec);
    }

    // generic setter methods
    public void addToContentList(String spec, IDLObject content) 
	throws IDLflexException
    {
        if (spec.equalsIgnoreCase("elements")) 
	    elements.addElement(content);
        else super.addToContentList(spec, content);
    }



    public boolean is_a(String spec, boolean inherit) {
	if(spec.equalsIgnoreCase("StructObj")) return true;
	if(inherit) return super.is_a(spec, inherit);
	else return false;
    }
}
