/*
 * IDLflex -- A flexible, generic code generator for CORBA IDL
 *
 * Copyright (C) 2000  Hans Reiser, AspectIX Research Team
 *                     University of Erlangen-Nuernberg, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 * The full text of the license can be found in the LICENSE.txt file.
 *
 * $Id: UnionMemberObj.java,v 1.4 2001/09/07 09:20:45 reiser Exp $
 */

package org.aspectix.IDLflex.IRObj;

import java.lang.Object;
import org.omg.CORBA.*;
import org.aspectix.IDLflex.IDLflexException;
import java.util.Vector;

/** implementation object for union members
 *<ul>
 * <li><em>getAttribute("unique")</em>
 *         is true, if this entry has just one label
 * <li><em>getAttribute("isdefault")></em>
 *         is true, if this label is the default label
 * <li><em>getName("discr")/getValue("discr")</em>
 *         numeric value of discriminator label value
 * <li><em>getValue("discrany")</em>
 *         label stored in an any
 * <li><em>getContent("BASE")</em>
 *         base type of this member entry
 * <li><em>getContent("SUPER")</em>
 *         returns the surrounding UnionObj object
 * <li><em>getContentList("MEMBERS")</em>
 *         for an UnionObj's UNIQUE list, this MEMBERS list containes
 *         all labels for this definition entry
 *</ul>
 */
public class UnionMemberObj extends IDLObject {
    UnionObj dad;
    IDLObject type;
    Any label;
    boolean isdef;
    Vector labels;
    int cnt, offset;
    UnionMember[] um;

    public UnionMemberObj(UnionMember[] um, int offset, int cnt) {
	super( um[offset].type_def );
	name = um[offset].name;
	label = um[offset].label;
	this.offset=offset; this.cnt=cnt; this.um=um;
	
	isdef=false;
	try {
	    if( um[offset].label.type().kind().equals(TCKind.tk_octet) )
		if( um[offset].label.extract_octet()==0 ) isdef=true;
	} catch(BAD_OPERATION e) {}
    }

    public void init_members() throws IDLflexException {
	super.init_members();

	if(cnt>0) {
	    labels = new Vector();
	    for(int i=0; i<cnt; i++) {
		IDLObject idlo = new UnionMemberObj(um, offset+i, 0);
		((UnionMemberObj)idlo).dad = dad;
		idlo.init_members();
		labels.addElement(idlo);
	    }
	} else labels=null;

	if(referenceObject!=null) {
	    type = ObjectCreator.get( referenceObject );
	} else {
	    System.err.println("UnionMemberObj: init: no reference Object");
	}
    }

    public boolean getAttribute(String spec) throws IDLflexException {
	spec = spec.toLowerCase();
	if(spec.equalsIgnoreCase("unique")) 
	    return (labels==null||labels.size()<=1);
	if(spec.equalsIgnoreCase("isdefault")) return isdef;
	return super.getAttribute(spec);
    }

    public String getName(String spec) throws IDLflexException
    {
	long labelnr = UnionObj.getLabelNumber(label);
	if(spec.equalsIgnoreCase("discr")) return "" + labelnr;
	else return super.getName(spec);
    }

    public Object getValue(String spec) throws IDLflexException
    {
	long labelnr = UnionObj.getLabelNumber(label);
	if(spec.equalsIgnoreCase("discr")) return new Long(labelnr);
	if(spec.equalsIgnoreCase("discrany")) return label;
	return super.getValue(spec);
    }
	
    public IDLObject getContent(String spec) throws IDLflexException {
	if(spec.equalsIgnoreCase("BASE")) return type;
	else if(spec.equalsIgnoreCase("SUPER")) return dad;
	else return super.getContent(spec);
    } 
    public IDLObject[] getContentList(String spec) throws IDLflexException {
	if(spec.equalsIgnoreCase("ELEMENTS")) 
	    return VectorToIDLObjects(labels);
	else return super.getContentList(spec);
    }

    public void addToContentList(String spec, IDLObject content)
	throws IDLflexException 
    {
        if (spec.equalsIgnoreCase("elements")) 
	    labels.addElement(content);
        else super.addToContentList(spec, content);
    }


    public boolean is_a(String spec, boolean inherit) {
	if(spec.equalsIgnoreCase("UnionMemberObj")) return true;
	if(inherit) return super.is_a(spec, inherit);
	else return false;
    }
}
